# Definition of the flow network

param n;					# N. of nodes
param m;					# N. of arcs

set V := 1 .. n;			# Node set
set A within {V cross V};	# Arc set
set K := 1 .. m;			# Set of commodities

param c{A,K};				# Unit flow cost for each arc
param b{V,K};				# Supply/demand for each node
param u{A};					# Capacity for each arc 

### Master problem

# Subsets of columns, one subset for each commodity
param p{K} default 1;				# N. of columns for each commodity
set P{k in K} := 1 .. p[k];			# Column subset for each commodity
param a{A,k in K,P[k]} default 0;	# Arcs in each path (column)

# Variables
var f{k in K, P[k]} >=0;	# N. of flow units along each path
var dummy{K} >= 0;			# N. of flow units along the dummy path

minimize master: 
    sum{k in K} 10000 * dummy[k] 
  + sum{k in K} sum{q in P[k]} (sum{(i,j) in A} c[i,j,k] * a[i,j,k,q]) * f[k,q];

subject to cap{(i,j) in A}: 
  sum{k in K, q in P[k]} - a[i,j,k,q] * f[k,q] >= - u[i,j];

subject to flow{k in K}: 
  dummy[k] + sum{q in P[k]} f[k,q] >= sum{i in V: b[i,k] > 0} b[i,k];

### Pricing sub-problem (with the integrality property)

# Dual variables
param lambda{A} default 0;
param mu{K} default 0;

# Unit supply/demand for each node and commodity
param d{i in V,k in K} := 
	if (b[i,k] > 0) then 1 
	else  
		if (b[i,k] < 0) then -1 
		else 0;

# Arc variables (binary, but relaxed for the I.P.)
var x{A} >=0, <=1;

minimize rc{k in K}: 
  sum{(i,j) in A} (c[i,j,k] + lambda[i,j]) * x[i,j] - mu[k];
  
subject to flowcons{k in K, i in V}: 
  sum{(l,j) in A: l == i} x[i,j] = sum{(j,l) in A: l == i} x[j,i] + d[i,k];

###########

problem RMP: master, cap, flow, f, dummy;

problem PP{k in K}: rc[k], {i in V}flowcons[k,i], x;
